@@ -21,6 +21,7 @@
 #include <QDebug>
 #include <QStandardPaths>
 #include <QFile>
+#include <QTemporaryDir>
 
 #include <kfileplacesview.h>
 #include <kfileplacesmodel.h>
@@ -45,10 +46,14 @@ private Q_SLOTS:
 
     void testUrlChanged_data();
     void testUrlChanged();
+private:
+    QTemporaryDir m_tmpHome;
 };
 
 void KFilePlacesViewTest::initTestCase()
 {
+    QVERIFY(m_tmpHome.isValid());
+    qputenv("HOME", m_tmpHome.path().toUtf8());
     qputenv("KDE_FORK_SLAVES", "yes"); // to avoid a runtime dependency on klauncher
     QStandardPaths::setTestModeEnabled(true);
 
@@ -74,20 +79,17 @@ void KFilePlacesViewTest::testUrlChanged_data()
 
     const QDate currentDate = QDate::currentDate();
     const QDate yesterdayDate = currentDate.addDays(-1);
-    QTest::newRow("Today") << 4 << QStringLiteral("timeline:/today");
-    QTest::newRow("Yesterday") << 5 << QString("timeline:/%1-%2/%1-%2-%3")
+    QTest::newRow("Today") << 3 << QStringLiteral("timeline:/today");
+    QTest::newRow("Yesterday") << 4 << QString("timeline:/%1-%2/%1-%2-%3")
                                   .arg(yesterdayDate.year())
                                   .arg(yesterdayDate.month(), 2, 10, QChar('0'))
                                   .arg(yesterdayDate.day(), 2, 10, QChar('0'));
 
     // search
-    const QString baloonurl = QStringLiteral("baloosearch:?json=%7B%22dayFilter%22: 0, %22monthFilter%22: 0, %22yearFilter%22: 0, %22type%22: [ %22$TYPE$%22]%7D");
-    const QString typeToReplace = QStringLiteral("$TYPE$");
-
-    QTest::newRow("Documents") << 6 << QString(baloonurl).replace(typeToReplace, QStringLiteral("Document"));
-    QTest::newRow("Images") << 7 << QString(baloonurl).replace(typeToReplace, QStringLiteral("Image"));
-    QTest::newRow("Audio Files") << 8 << QString(baloonurl).replace(typeToReplace, QStringLiteral("Audio"));
-    QTest::newRow("Videos") << 9 << QString(baloonurl).replace(typeToReplace, QStringLiteral("Video"));
+    QTest::newRow("Documents") << 5 << QStringLiteral("baloosearch:/documents");
+    QTest::newRow("Images") << 6 << QStringLiteral("baloosearch:/images");
+    QTest::newRow("Audio Files") << 7 << QStringLiteral("baloosearch:/audio");
+    QTest::newRow("Videos") << 8 << QStringLiteral("baloosearch:/videos");
 }
 
 void KFilePlacesViewTest::testUrlChanged()
@@ -97,6 +99,7 @@ void KFilePlacesViewTest::testUrlChanged()
 
     KFilePlacesView pv;
     pv.show();
+    pv.activateWindow();
     pv.setModel(new KFilePlacesModel());
     QVERIFY(QTest::qWaitForWindowActive(&pv));
 
