@@ -23,11 +23,13 @@
 #include <kiconloader.h>
 #include "kdirmodel.h"
 #include "kdiroperatordetailview_p.h"
+#include "kdiroperatoriconview_p.h"
 #include "kdirsortfilterproxymodel.h"
 #include "kfileitem.h"
 #include "kfilemetapreview_p.h"
 #include "kpreviewwidgetbase.h"
 #include "knewfilemenu.h"
+#include <kurlmimedata.h>
 #include "../pathhelpers_p.h"
 
 #include <config-kiofilewidgets.h>
@@ -37,9 +39,7 @@
 #include <QHeaderView>
 #include <QListView>
 #include <QMenu>
-#include <QMimeData>
 #include <QProgressBar>
-#include <QScrollBar>
 #include <QSplitter>
 #include <QWheelEvent>
 #include <QTimer>
@@ -69,96 +69,6 @@ template class QHash<QString, KFileItem>;
 // sorting mode.
 static const int QDirSortMask = QDir::SortByMask | QDir::Type;
 
-/**
- * Default icon view for KDirOperator using
- * custom view options.
- */
-class KDirOperatorIconView : public QListView
-{
-    Q_OBJECT
-public:
-    KDirOperatorIconView(KDirOperator *dirOperator, QWidget *parent = nullptr);
-    virtual ~KDirOperatorIconView();
-
-protected:
-    QStyleOptionViewItem viewOptions() const override;
-    void dragEnterEvent(QDragEnterEvent *event) override;
-    void mousePressEvent(QMouseEvent *event) override;
-    void wheelEvent(QWheelEvent *event) override;
-
-private:
-    KDirOperator *ops;
-};
-
-KDirOperatorIconView::KDirOperatorIconView(KDirOperator *dirOperator, QWidget *parent) :
-    QListView(parent),
-    ops(dirOperator)
-{
-    setViewMode(QListView::IconMode);
-    setFlow(QListView::TopToBottom);
-    setResizeMode(QListView::Adjust);
-    setSpacing(0);
-    setMovement(QListView::Static);
-    setDragDropMode(QListView::DragOnly);
-    setVerticalScrollMode(QListView::ScrollPerPixel);
-    setHorizontalScrollMode(QListView::ScrollPerPixel);
-    setEditTriggers(QAbstractItemView::NoEditTriggers);
-    setWordWrap(true);
-    setIconSize(QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall));
-}
-
-KDirOperatorIconView::~KDirOperatorIconView()
-{
-}
-
-QStyleOptionViewItem KDirOperatorIconView::viewOptions() const
-{
-    QStyleOptionViewItem viewOptions = QListView::viewOptions();
-    viewOptions.showDecorationSelected = true;
-    viewOptions.decorationPosition = ops->decorationPosition();
-    if (viewOptions.decorationPosition == QStyleOptionViewItem::Left) {
-        viewOptions.displayAlignment = Qt::AlignLeft | Qt::AlignVCenter;
-    } else {
-        viewOptions.displayAlignment = Qt::AlignCenter;
-    }
-
-    return viewOptions;
-}
-
-void KDirOperatorIconView::dragEnterEvent(QDragEnterEvent *event)
-{
-    if (event->mimeData()->hasUrls()) {
-        event->acceptProposedAction();
-    }
-}
-
-void KDirOperatorIconView::mousePressEvent(QMouseEvent *event)
-{
-    if (!indexAt(event->pos()).isValid()) {
-        const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
-        if (!(modifiers & Qt::ShiftModifier) && !(modifiers & Qt::ControlModifier)) {
-            clearSelection();
-        }
-    }
-
-    QListView::mousePressEvent(event);
-}
-
-void KDirOperatorIconView::wheelEvent(QWheelEvent *event)
-{
-    QListView::wheelEvent(event);
-
-    // apply the vertical wheel event to the horizontal scrollbar, as
-    // the items are aligned from left to right
-    if (event->orientation() == Qt::Vertical) {
-        QWheelEvent horizEvent(event->pos(),
-                               event->delta(),
-                               event->buttons(),
-                               event->modifiers(),
-                               Qt::Horizontal);
-        QApplication::sendEvent(horizontalScrollBar(), &horizEvent);
-    }
-}
 
 void KDirOperator::keyPressEvent(QKeyEvent *e)
 {
@@ -194,12 +104,24 @@ public:
 
     KFile::FileView allViews();
 
+    QMetaObject::Connection m_connection;
+
+    // A pair to store zoom settings for view kinds
+    struct ZoomSettingsForView {
+        QString name;
+        int defaultValue;
+    };
+
     // private slots
     void _k_slotDetailedView();
     void _k_slotSimpleView();
     void _k_slotTreeView();
     void _k_slotDetailedTreeView();
+    void _k_slotIconsView();
+    void _k_slotCompactView();
+    void _k_slotDetailsView();
     void _k_slotToggleHidden(bool);
+    void _k_slotToggleAllowExpansion(bool);
     void _k_togglePreview(bool);
     void _k_toggleInlinePreviews(bool);
     void _k_slotOpenFileManager();
@@ -209,6 +131,9 @@ public:
     void _k_slotSortByType();
     void _k_slotSortReversed(bool doReverse);
     void _k_slotToggleDirsFirst();
+    void _k_slotToggleIconsView();
+    void _k_slotToggleCompactView();
+    void _k_slotToggleDetailsView();
     void _k_slotToggleIgnoreCase();
     void _k_slotStarted();
     void _k_slotProgress(int);
@@ -230,8 +155,9 @@ public:
     void _k_slotItemsChanged();
     void _k_slotDirectoryCreated(const QUrl &);
 
-    void updateListViewGrid();
     int iconSizeForViewType(QAbstractItemView *itemView) const;
+    void writeIconZoomSettingsIfNeeded();
+    ZoomSettingsForView zoomSettingsForViewForView() const;
 
     // private members
     KDirOperator * const parent;
@@ -414,6 +340,7 @@ KDirOperator::KDirOperator(const QUrl &_url, QWidget *parent) :
     d->updateSorting(QDir::Name | QDir::DirsFirst);
 
     setFocusPolicy(Qt::WheelFocus);
+    setAcceptDrops(true);
 }
 
 KDirOperator::~KDirOperator()
@@ -541,7 +468,7 @@ KFileItemList KDirOperator::selectedItems() const
     const QItemSelection selection = d->proxyModel->mapSelectionToSource(d->itemView->selectionModel()->selection());
 
     const QModelIndexList indexList = selection.indexes();
-    foreach (const QModelIndex &index, indexList) {
+    for (const QModelIndex &index : indexList) {
         KFileItem item = d->dirModel->itemForIndex(index);
         if (!item.isNull()) {
             itemList.append(item);
@@ -594,28 +521,48 @@ KFile::FileView KDirOperator::Private::allViews()
 
 void KDirOperator::Private::_k_slotDetailedView()
 {
+    // save old zoom settings
+    writeIconZoomSettingsIfNeeded();
+
     KFile::FileView view = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::Detail);
     parent->setView(view);
 }
 
 void KDirOperator::Private::_k_slotSimpleView()
 {
+    // save old zoom settings
+    writeIconZoomSettingsIfNeeded();
+
     KFile::FileView view = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::Simple);
     parent->setView(view);
 }
 
 void KDirOperator::Private::_k_slotTreeView()
 {
+    // save old zoom settings
+    writeIconZoomSettingsIfNeeded();
+
     KFile::FileView view = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::Tree);
     parent->setView(view);
 }
 
 void KDirOperator::Private::_k_slotDetailedTreeView()
 {
+    // save old zoom settings
+    writeIconZoomSettingsIfNeeded();
+
     KFile::FileView view = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::DetailTree);
     parent->setView(view);
 }
 
+void KDirOperator::Private::_k_slotToggleAllowExpansion(bool allow) {
+    KFile::FileView view = KFile::Detail;
+    if (allow) {
+        view = KFile::DetailTree;
+    }
+    parent->setView(view);
+}
+
 void KDirOperator::Private::_k_slotToggleHidden(bool show)
 {
     dirLister->setShowingDotFiles(show);
@@ -708,6 +655,48 @@ void KDirOperator::Private::_k_slotToggleDirsFirst()
     updateSorting(s);
 }
 
+void KDirOperator::Private::_k_slotIconsView()
+{
+    // save old zoom settings
+    writeIconZoomSettingsIfNeeded();
+
+    // Put the icons on top
+    actionCollection->action(QStringLiteral("decorationAtTop"))->setChecked(true);
+    decorationPosition = QStyleOptionViewItem::Top;
+
+    // Switch to simple view
+    KFile::FileView fileView = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::Simple);
+    parent->setView(fileView);
+}
+
+void KDirOperator::Private::_k_slotCompactView()
+{
+    // save old zoom settings
+    writeIconZoomSettingsIfNeeded();
+
+    // Put the icons on the side
+    actionCollection->action(QStringLiteral("decorationAtLeft"))->setChecked(true);
+    decorationPosition = QStyleOptionViewItem::Left;
+
+    // Switch to simple view
+    KFile::FileView fileView = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::Simple);
+    parent->setView(fileView);
+}
+
+void KDirOperator::Private::_k_slotDetailsView()
+{
+    // save old zoom settings
+    writeIconZoomSettingsIfNeeded();
+
+    KFile::FileView view;
+    if (actionCollection->action(QStringLiteral("allow expansion"))->isChecked()) {
+        view = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::DetailTree);
+    } else {
+        view = static_cast<KFile::FileView>((viewKind & ~allViews()) | KFile::Detail);
+    }
+    parent->setView(view);
+}
+
 void KDirOperator::Private::_k_slotToggleIgnoreCase()
 {
     // TODO: port to Qt4's QAbstractItemView
@@ -916,7 +905,6 @@ void KDirOperator::setIconsZoom(int _value)
     const int maxSize = KIconLoader::SizeEnormous - KIconLoader::SizeSmall;
     const int val = (maxSize * value / 100) + KIconLoader::SizeSmall;
     d->itemView->setIconSize(QSize(val, val));
-    d->updateListViewGrid();
     d->previewGenerator->updatePreviews();
 
     emit currentIconSizeChanged(value);
@@ -1379,6 +1367,77 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
         }
     }
     break;
+    case QEvent::DragEnter: {
+        // Accepts drops of one file or folder only
+        QDragEnterEvent *evt = static_cast<QDragEnterEvent *>(event);
+        const QList<QUrl> urls = KUrlMimeData::urlsFromMimeData(evt->mimeData(), KUrlMimeData::DecodeOptions::PreferLocalUrls);
+
+        // only one file/folder can be dropped at the moment
+        if (urls.size() != 1) {
+            evt->ignore();
+
+        } else {
+            // mimetype filtering
+            bool mimeFilterPass = true;
+            const QStringList mimeFilters = d->dirLister->mimeFilters();
+
+            if (mimeFilters.size() > 1) {
+                mimeFilterPass = false;
+                const QUrl url = urls.constFirst();
+
+                QMimeDatabase mimeDataBase;
+                QMimeType fileMimeType = mimeDataBase.mimeTypeForUrl(url);
+
+                QRegularExpression regex;
+                for (const auto& mimeFilter : mimeFilters) {
+                    regex.setPattern(mimeFilter);
+                    if (regex.match(fileMimeType.name()).hasMatch()) {   // matches!
+                        mimeFilterPass = true;
+                        break;
+                    }
+                }
+            }
+
+            event->setAccepted(mimeFilterPass);
+        }
+
+        return true;
+    }
+    case QEvent::Drop: {
+        QDropEvent *evt = static_cast<QDropEvent *>(event);
+        const QList<QUrl> urls = KUrlMimeData::urlsFromMimeData(evt->mimeData(), KUrlMimeData::DecodeOptions::PreferLocalUrls);
+
+        const QUrl url = urls.constFirst();
+
+        // stat the url to get details
+        KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
+        job->exec();
+
+        setFocus();
+
+        KIO::UDSEntry entry = job->statResult();
+
+        if (entry.isDir()) {
+            // if this was a directory
+            setUrl(url, false);
+        } else {
+            // if the current url is not known
+            if (d->dirLister->findByUrl(url).isNull()) {
+                setUrl(url.adjusted(QUrl::RemoveFilename), false);
+
+                // Will set the current item once loading has finished
+                auto urlSetterClosure = [this, url](){
+                    setCurrentItem(url);
+                    QObject::disconnect(d->m_connection);
+                };
+                d->m_connection = connect(this, &KDirOperator::finishedLoading, this, urlSetterClosure);
+            } else {
+                setCurrentItem(url);
+            }
+        }
+        evt->accept();
+        return true;
+    }
     default:
         break;
     }
@@ -1455,18 +1514,23 @@ QAbstractItemView *KDirOperator::createView(QWidget *parent, KFile::FileView vie
         detailView->setViewMode(viewKind);
         itemView = detailView;
     } else {
-        itemView = new KDirOperatorIconView(this, parent);
+        itemView = new KDirOperatorIconView(parent, decorationPosition());
     }
 
     return itemView;
 }
 
-void KDirOperator::setAcceptDrops(bool b)
+void KDirOperator::setAcceptDrops(bool acceptsDrops)
 {
-    // TODO:
-    //if (d->fileView)
-    //   d->fileView->widget()->setAcceptDrops(b);
-    QWidget::setAcceptDrops(b);
+    QWidget::setAcceptDrops(acceptsDrops);
+    if (view()) {
+        view()->setAcceptDrops(acceptsDrops);
+        if (acceptsDrops) {
+            view()->installEventFilter(this);
+        } else {
+            view()->removeEventFilter(this);
+        }
+    }
 }
 
 void KDirOperator::setDropOptions(int options)
@@ -1503,6 +1567,11 @@ void KDirOperator::setView(KFile::FileView viewKind)
     QAbstractItemView *newView = createView(this, viewKind);
     setView(newView);
 
+    if (acceptDrops()) {
+        newView->setAcceptDrops(true);
+        newView->installEventFilter(this);
+    }
+
     d->_k_togglePreview(preview);
 }
 
@@ -1593,7 +1662,6 @@ void KDirOperator::setView(QAbstractItemView *view)
     connect(d->itemView, SIGNAL(entered(QModelIndex)),
             this, SLOT(_k_triggerPreview(QModelIndex)));
 
-    updateViewActions();
     d->splitter->insertWidget(0, d->itemView);
 
     d->splitter->resize(size());
@@ -1638,6 +1706,7 @@ void KDirOperator::setView(QAbstractItemView *view)
 
     // ensure we change everything needed
     d->_k_slotChangeDecorationPosition();
+    updateViewActions();
 
     emit viewChanged(view);
 
@@ -1758,7 +1827,7 @@ void KDirOperator::setCurrentItems(const QList<QUrl> &urls)
     }
 
     KFileItemList itemList;
-    foreach (const QUrl &url, urls) {
+    for (const QUrl &url : urls) {
         KFileItem item = d->dirLister->findByUrl(url);
         if (d->shouldFetchForItems && item.isNull()) {
             d->itemsToBeSetAsCurrent << url;
@@ -1783,7 +1852,7 @@ void KDirOperator::setCurrentItems(const KFileItemList &items)
     if (selModel) {
         selModel->clear();
         QModelIndex proxyIndex;
-        foreach (const KFileItem &item, items) {
+        for (const KFileItem &item : items) {
             if (!item.isNull()) {
                 const QModelIndex dirIndex = d->dirModel->indexForItem(item);
                 proxyIndex = d->proxyModel->mapFromSource(dirIndex);
@@ -1826,7 +1895,7 @@ void KDirOperator::prepareCompletionObjects()
 
     if (d->completeListDirty) {   // create the list of all possible completions
         const KFileItemList itemList = d->dirLister->items();
-        foreach (const KFileItem &item, itemList) {
+        for (const KFileItem &item : itemList) {
             d->completion.addItem(item.name());
             if (item.isDir()) {
                 d->dirCompletion.addItem(item.name());
@@ -1915,6 +1984,28 @@ void KDirOperator::setupActions()
     d->actionCollection->addAction(QStringLiteral("dirs first"), dirsFirstAction);
     connect(dirsFirstAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotToggleDirsFirst()));
 
+    // View modes that match those of Dolphin
+    KToggleAction *iconsViewAction = new KToggleAction(i18n("Icons View"), this);
+    iconsViewAction->setIcon(QIcon::fromTheme(QStringLiteral("view-list-icons")));
+    d->actionCollection->addAction(QStringLiteral("icons view"), iconsViewAction);
+    connect(iconsViewAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotIconsView()));
+
+    KToggleAction *compactViewAction = new KToggleAction(i18n("Compact View"), this);
+    compactViewAction->setIcon(QIcon::fromTheme(QStringLiteral("view-list-details")));
+    d->actionCollection->addAction(QStringLiteral("compact view"), compactViewAction);
+    connect(compactViewAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotCompactView()));
+
+    KToggleAction *detailsViewAction = new KToggleAction(i18n("Details View"), this);
+    detailsViewAction->setIcon(QIcon::fromTheme(QStringLiteral("view-list-tree")));
+    d->actionCollection->addAction(QStringLiteral("details view"), detailsViewAction);
+    connect(detailsViewAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotDetailsView()));
+
+    QActionGroup *viewModeGroup = new QActionGroup(this);
+    viewModeGroup->setExclusive(true);
+    iconsViewAction->setActionGroup(viewModeGroup);
+    compactViewAction->setActionGroup(viewModeGroup);
+    detailsViewAction->setActionGroup(viewModeGroup);
+
     QActionGroup *sortGroup = new QActionGroup(this);
     byNameAction->setActionGroup(sortGroup);
     bySizeAction->setActionGroup(sortGroup);
@@ -1936,6 +2027,7 @@ void KDirOperator::setupActions()
     d->decorationMenu->addAction(topAction);
 
     QActionGroup *decorationGroup = new QActionGroup(this);
+    decorationGroup->setExclusive(true);
     d->leftAction->setActionGroup(decorationGroup);
     topAction->setActionGroup(decorationGroup);
 
@@ -1965,12 +2057,16 @@ void KDirOperator::setupActions()
     treeAction->setActionGroup(viewGroup);
     detailedTreeAction->setActionGroup(viewGroup);
 
+    KToggleAction *allowExpansionAction = new KToggleAction(i18n("Allow Expansion in Details View"), this);
+    d->actionCollection->addAction(QStringLiteral("allow expansion"), allowExpansionAction);
+    connect(allowExpansionAction, SIGNAL(toggled(bool)), SLOT(_k_slotToggleAllowExpansion(bool)));
+
     KToggleAction *showHiddenAction = new KToggleAction(i18n("Show Hidden Files"), this);
     d->actionCollection->addAction(QStringLiteral("show hidden"), showHiddenAction);
     showHiddenAction->setShortcuts({Qt::ALT + Qt::Key_Period, Qt::CTRL + Qt::Key_H, Qt::Key_F8});
     connect(showHiddenAction, SIGNAL(toggled(bool)), SLOT(_k_slotToggleHidden(bool)));
 
-    KToggleAction *previewAction = new KToggleAction(i18n("Show Aside Preview"), this);
+    KToggleAction *previewAction = new KToggleAction(i18n("Show Preview Panel"), this);
     d->actionCollection->addAction(QStringLiteral("preview"), previewAction);
     previewAction->setShortcut(Qt::Key_F11);
     connect(previewAction, SIGNAL(toggled(bool)),
@@ -2102,6 +2198,11 @@ void KDirOperator::updateViewActions()
     d->actionCollection->action(QStringLiteral("detailed view"))->setChecked(KFile::isDetailView(fv));
     d->actionCollection->action(QStringLiteral("tree view"))->setChecked(KFile::isTreeView(fv));
     d->actionCollection->action(QStringLiteral("detailed tree view"))->setChecked(KFile::isDetailTreeView(fv));
+
+    // dolphin style views
+    d->actionCollection->action(QStringLiteral("icons view"))->setChecked(KFile::isSimpleView(fv) && d->decorationPosition == QStyleOptionViewItem::Top);
+    d->actionCollection->action(QStringLiteral("compact view"))->setChecked(KFile::isSimpleView(fv) && d->decorationPosition == QStyleOptionViewItem::Left);
+    d->actionCollection->action(QStringLiteral("details view"))->setChecked(KFile::isDetailTreeView(fv) || KFile::isDetailView(fv));
 }
 
 void KDirOperator::readConfig(const KConfigGroup &configGroup)
@@ -2133,6 +2234,11 @@ void KDirOperator::readConfig(const KConfigGroup &configGroup)
         d->dirLister->setShowingDotFiles(true);
     }
 
+    if (configGroup.readEntry(QStringLiteral("Allow Expansion"),
+                              DefaultShowHidden)) {
+        d->actionCollection->action(QStringLiteral("allow expansion"))->setChecked(true);
+    }
+
     QDir::SortFlags sorting = QDir::Name;
     if (configGroup.readEntry(QStringLiteral("Sort directories first"),
                               DefaultDirsFirst)) {
@@ -2157,7 +2263,7 @@ void KDirOperator::readConfig(const KConfigGroup &configGroup)
     if (d->inlinePreviewState == Private::NotForced) {
         d->showPreviews = configGroup.readEntry(QStringLiteral("Show Inline Previews"), true);
     }
-    QStyleOptionViewItem::Position pos = (QStyleOptionViewItem::Position) configGroup.readEntry(QStringLiteral("Decoration position"), (int) QStyleOptionViewItem::Left);
+    QStyleOptionViewItem::Position pos = (QStyleOptionViewItem::Position) configGroup.readEntry(QStringLiteral("Decoration position"), (int) QStyleOptionViewItem::Top);
     setDecorationPosition(pos);
 }
 
@@ -2205,6 +2311,9 @@ void KDirOperator::writeConfig(KConfigGroup &configGroup)
     configGroup.writeEntry(QStringLiteral("Show hidden files"),
                            d->actionCollection->action(QStringLiteral("show hidden"))->isChecked());
 
+    configGroup.writeEntry(QStringLiteral("Allow Expansion"),
+                           d->actionCollection->action(QStringLiteral("allow expansion"))->isChecked());
+
     KFile::FileView fv = static_cast<KFile::FileView>(d->viewKind);
     QString style;
     if (KFile::isDetailView(fv)) {
@@ -2220,16 +2329,20 @@ void KDirOperator::writeConfig(KConfigGroup &configGroup)
 
     if (d->inlinePreviewState == Private::NotForced) {
         configGroup.writeEntry(QStringLiteral("Show Inline Previews"), d->showPreviews);
-        if (qobject_cast<QListView *>(d->itemView)) {
-            configGroup.writeEntry(QStringLiteral("listViewIconSize"), d->iconsZoom);
-        } else {
-            configGroup.writeEntry(QStringLiteral("detailedViewIconSize"), d->iconsZoom);
-        }
+        d->writeIconZoomSettingsIfNeeded();
     }
 
     configGroup.writeEntry(QStringLiteral("Decoration position"), (int) d->decorationPosition);
 }
 
+void KDirOperator::Private::writeIconZoomSettingsIfNeeded() {
+     // must match behavior of iconSizeForViewType
+     if (configGroup && itemView) {
+         ZoomSettingsForView zoomSettings = zoomSettingsForViewForView();
+         configGroup->writeEntry(zoomSettings.name, iconsZoom);
+     }
+}
+
 void KDirOperator::resizeEvent(QResizeEvent *)
 {
     // resize the splitter and assure that the width of
@@ -2255,8 +2368,6 @@ void KDirOperator::resizeEvent(QResizeEvent *)
         // might be reparented into a statusbar
         d->progressBar->move(2, height() - d->progressBar->height() - 2);
     }
-
-    d->updateListViewGrid();
 }
 
 void KDirOperator::setOnlyDoubleClickSelectsFiles(bool enable)
@@ -2501,7 +2612,7 @@ void KDirOperator::Private::_k_slotChangeDecorationPosition()
         return;
     }
 
-    QListView *view = qobject_cast<QListView *>(itemView);
+    KDirOperatorIconView *view = qobject_cast<KDirOperatorIconView *>(itemView);
 
     if (!view) {
         return;
@@ -2510,15 +2621,11 @@ void KDirOperator::Private::_k_slotChangeDecorationPosition()
     const bool leftChecked = actionCollection->action(QStringLiteral("decorationAtLeft"))->isChecked();
 
     if (leftChecked) {
-        decorationPosition = QStyleOptionViewItem::Left;
-        view->setFlow(QListView::TopToBottom);
+        view->setDecorationPosition(QStyleOptionViewItem::Left);
     } else {
-        decorationPosition = QStyleOptionViewItem::Top;
-        view->setFlow(QListView::LeftToRight);
+        view->setDecorationPosition(QStyleOptionViewItem::Top);
     }
 
-    updateListViewGrid();
-
     itemView->update();
 }
 
@@ -2569,62 +2676,36 @@ void KDirOperator::Private::_k_slotItemsChanged()
     completeListDirty = true;
 }
 
-void KDirOperator::Private::updateListViewGrid()
-{
-    if (!itemView) {
-        return;
-    }
-
-    QListView *view = qobject_cast<QListView *>(itemView);
-
-    if (!view) {
-        return;
-    }
-
-    const bool leftChecked = actionCollection->action(QStringLiteral("decorationAtLeft"))->isChecked();
-
-    if (leftChecked) {
-        view->setGridSize(QSize());
-        KFileItemDelegate *delegate = qobject_cast<KFileItemDelegate *>(view->itemDelegate());
-        if (delegate) {
-            delegate->setMaximumSize(QSize());
-        }
-    } else {
-        const QFontMetrics metrics(itemView->viewport()->font());
-
-        const int height = itemView->iconSize().height() + metrics.height() * 2.5;
-        const int minWidth = qMax(height, metrics.height() * 5);
-
-        const int scrollBarWidth = itemView->verticalScrollBar()->sizeHint().width();
-
-        // Subtract 1 px to prevent flickering when resizing the window
-        // For Oxygen a column is missing after showing the dialog without resizing it,
-        // therefore subtract 4 more (scaled) pixels
-        const int viewPortWidth = itemView->contentsRect().width() - scrollBarWidth - 1 - 4 * itemView->devicePixelRatioF();
-        const int itemsInRow = qMax(1, viewPortWidth / minWidth);
-        const int remainingWidth = viewPortWidth - (minWidth * itemsInRow);
-        const int width = minWidth + (remainingWidth / itemsInRow);
-        
-        const QSize itemSize(width, height);
-
-        view->setGridSize(itemSize);
-        KFileItemDelegate *delegate = qobject_cast<KFileItemDelegate *>(view->itemDelegate());
-        if (delegate) {
-            delegate->setMaximumSize(itemSize);
-        }
-    }
-}
-
 int KDirOperator::Private::iconSizeForViewType(QAbstractItemView *itemView) const
 {
+    // must match behavior of writeIconZoomSettingsIfNeeded
     if (!itemView || !configGroup) {
         return 0;
     }
 
-    if (qobject_cast<QListView *>(itemView)) {
-        return configGroup->readEntry("listViewIconSize", 0);
+    ZoomSettingsForView ZoomSettingsForView = zoomSettingsForViewForView();
+    return configGroup->readEntry(ZoomSettingsForView.name, ZoomSettingsForView.defaultValue);
+}
+
+KDirOperator::Private::ZoomSettingsForView KDirOperator::Private::zoomSettingsForViewForView() const {
+     KFile::FileView fv = static_cast<KFile::FileView>(viewKind);
+     if (KFile::isSimpleView(fv)) {
+         if (decorationPosition == QStyleOptionViewItem::Top){
+             // Simple view decoration above, aka Icons View
+             // default to 43% aka 64px
+             return {QStringLiteral("iconViewIconSize"), 43};
+         } else {
+             // Simple view decoration left, aka compact view
+             // default to 15% aka 32px
+             return {QStringLiteral("listViewIconSize"), 15};
+         }
     } else {
-        return configGroup->readEntry("detailedViewIconSize", 0);
+         if (KFile::isTreeView(fv)) {
+             return {QStringLiteral("treeViewIconSize"), 0};
+         } else {
+             // DetailView and DetailTreeView
+             return {QStringLiteral("detailViewIconSize"), 0};
+         }
     }
 }
 
@@ -2687,4 +2768,3 @@ QStringList KDirOperator::supportedSchemes() const
 }
 
 #include "moc_kdiroperator.cpp"
-#include "kdiroperator.moc"
