@@ -43,14 +43,14 @@
 #include <cstdlib>
 #include <cstring>
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDir>
-#include <QtNetwork/QHostAddress>
-#include <QtNetwork/QNetworkProxy>
-#include <QtNetwork/QTcpSocket>
-#include <QtNetwork/QTcpServer>
-#include <QtNetwork/QSslSocket>
-#include <QtNetwork/QAuthenticator>
+#include <QCoreApplication>
+#include <QDir>
+#include <QHostAddress>
+#include <QNetworkProxy>
+#include <QTcpSocket>
+#include <QTcpServer>
+#include <QSslSocket>
+#include <QAuthenticator>
 #include <qmimedatabase.h>
 
 #include <QDebug>
@@ -66,9 +66,9 @@ Q_DECLARE_LOGGING_CATEGORY(KIO_FTP)
 Q_LOGGING_CATEGORY(KIO_FTP, "kf5.kio.kio_ftp", QtWarningMsg)
 
 #if HAVE_STRTOLL
-#define charToLongLong(a) strtoll(a, 0, 10)
+#define charToLongLong(a) strtoll(a, nullptr, 10)
 #else
-#define charToLongLong(a) strtol(a, 0, 10)
+#define charToLongLong(a) strtol(a, nullptr, 10)
 #endif
 
 #define FTP_LOGIN   "anonymous"
@@ -237,7 +237,7 @@ void Ftp::ftpCloseControlConnection()
     delete m_control;
     m_control = nullptr;
     m_cDataMode = 0;
-    m_bLoggedOn = false;    // logon needs control connction
+    m_bLoggedOn = false;    // logon needs control connection
     m_bTextMode = false;
     m_bBusy = false;
 }
@@ -257,7 +257,7 @@ const char *Ftp::ftpResponse(int iOffset)
         m_iRespCode = 0;
 
         if (!pTxt) {
-            return nullptr;    // avoid using a NULL when calling atoi.
+            return nullptr;    // avoid using a nullptr when calling atoi.
         }
 
         // If the server sends a multiline response starting with
@@ -399,7 +399,7 @@ bool Ftp::ftpOpenConnection(LoginMode loginMode)
             realURL.setPort(m_port);
         }
         if (m_initialPath.isEmpty()) {
-            m_initialPath = '/';
+            m_initialPath = QStringLiteral("/");
         }
         realURL.setPath(m_initialPath);
         qCDebug(KIO_FTP) << "User name changed! Redirecting to" << realURL;
@@ -471,8 +471,8 @@ bool Ftp::ftpOpenControlConnection(const QString &host, int port)
         port = 21;    // default FTP port
     }
     m_control = synchronousConnectToHost(host, port);
-    connect(m_control, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),
-            this, SLOT(proxyAuthentication(QNetworkProxy,QAuthenticator*)));
+    connect(m_control, &QAbstractSocket::proxyAuthenticationRequired,
+            this, &Ftp::proxyAuthentication);
     int iErrorCode = m_control->state() == QAbstractSocket::ConnectedState ? 0 : ERR_CANNOT_CONNECT;
 
     // on connect success try to read the server message...
@@ -607,14 +607,11 @@ bool Ftp::ftpLogin(bool *userChanged)
             }
         }
 
-        tempbuf = "USER ";
-        tempbuf += user.toLatin1();
+        tempbuf = "USER " + user.toLatin1();
         if (m_proxyURL.isValid()) {
-            tempbuf += '@';
-            tempbuf += m_host.toLatin1();
+            tempbuf += '@' + m_host.toLatin1();
             if (m_port > 0 && m_port != DEFAULT_FTP_PORT) {
-                tempbuf += ':';
-                tempbuf += QString::number(m_port).toLatin1();
+                tempbuf += ':' + QByteArray::number(m_port);
             }
         }
 
@@ -625,15 +622,14 @@ bool Ftp::ftpLogin(bool *userChanged)
         // Prompt user for login info if we do not
         // get back a "230" or "331".
         if (!loggedIn && !needPass) {
-            lastServerResponse = ftpResponse(0);
+            lastServerResponse = QString::fromUtf8(ftpResponse(0));
             qCDebug(KIO_FTP) << "Login failed: " << lastServerResponse;
             ++failedAuth;
             continue;  // Well we failed, prompt the user please!!
         }
 
         if (needPass) {
-            tempbuf = "PASS ";
-            tempbuf += pass.toLatin1();
+            tempbuf = "PASS " + pass.toLatin1();
             qCDebug(KIO_FTP) << "Sending Login password: " << "[protected]";
             loggedIn = (ftpSendCmd(tempbuf) && (m_iRespCode == 230));
         }
@@ -661,7 +657,7 @@ bool Ftp::ftpLogin(bool *userChanged)
         } else {
             // some servers don't let you login anymore
             // if you fail login once, so restart the connection here
-            lastServerResponse = ftpResponse(0);
+            lastServerResponse = QString::fromUtf8(ftpResponse(0));
             if (!ftpOpenControlConnection()) {
                 return false;
             }
@@ -703,12 +699,12 @@ bool Ftp::ftpLogin(bool *userChanged)
     }
 
     QString sTmp = remoteEncoding()->decode(ftpResponse(3));
-    int iBeg = sTmp.indexOf('"');
-    int iEnd = sTmp.lastIndexOf('"');
+    const int iBeg = sTmp.indexOf(QLatin1Char('"'));
+    const int iEnd = sTmp.lastIndexOf(QLatin1Char('"'));
     if (iBeg > 0 && iBeg < iEnd) {
         m_initialPath = sTmp.mid(iBeg + 1, iEnd - iBeg - 1);
-        if (m_initialPath[0] != '/') {
-            m_initialPath.prepend('/');
+        if (m_initialPath[0] != QLatin1Char('/')) {
+            m_initialPath.prepend(QLatin1Char('/'));
         }
         qCDebug(KIO_FTP) << "Initial path set to: " << m_initialPath;
         m_currentPath = m_initialPath;
@@ -724,11 +720,11 @@ void Ftp::ftpAutoLoginMacro()
         return;
     }
 
-    const QStringList list = macro.split('\n', QString::SkipEmptyParts);
+    const QStringList list = macro.split(QLatin1Char('\n'), QString::SkipEmptyParts);
 
     for (QStringList::const_iterator it = list.begin(); it != list.end(); ++it) {
         if ((*it).startsWith(QLatin1String("init"))) {
-            const QStringList list2 = macro.split('\\', QString::SkipEmptyParts);
+            const QStringList list2 = macro.split(QLatin1Char('\\'), QString::SkipEmptyParts);
             it = list2.begin();
             ++it;  // ignore the macro name
 
@@ -776,8 +772,7 @@ bool Ftp::ftpSendCmd(const QByteArray &cmd, int maxretries)
 #endif
 
     // Send the message...
-    QByteArray buf = cmd;
-    buf += "\r\n";      // Yes, must use CR/LF - see http://cr.yp.to/ftp/request.html
+    const QByteArray buf = cmd + "\r\n";      // Yes, must use CR/LF - see http://cr.yp.to/ftp/request.html
     int num = m_control->write(buf);
     while (m_control->bytesToWrite() && m_control->waitForBytesWritten()) {}
 
@@ -892,7 +887,7 @@ int Ftp::ftpOpenPASVDataConnection()
 
     // we ignore the host part on purpose for two reasons
     // a) it might be wrong anyway
-    // b) it would make us being suceptible to a port scanning attack
+    // b) it would make us being susceptible to a port scanning attack
 
     // now connect the data socket ...
     quint16 port = i[4] << 8 | i[5];
@@ -1079,16 +1074,15 @@ bool Ftp::ftpOpenCommand(const char *_command, const QString &_path, char _mode,
     QString errormessage;
 
     if (!_path.isEmpty()) {
-        tmp += ' ';
-        tmp += remoteEncoding()->encode(ftpCleanPath(_path));
+        tmp += ' ' + remoteEncoding()->encode(ftpCleanPath(_path));
     }
 
     if (!ftpSendCmd(tmp) || (m_iRespType != 1)) {
         if (_offset > 0 && qstrcmp(_command, "retr") == 0 && (m_iRespType == 4)) {
             errorcode = ERR_CANNOT_RESUME;
         }
-        // The error here depends on the command
-        errormessage = _path;
+        // The error code here depends on the command
+        errormessage = _path + i18n("\nThe server said: \"%1\"", QString::fromUtf8(ftpResponse(0)).trimmed());
     }
 
     else {
@@ -1204,22 +1198,20 @@ bool Ftp::ftpRename(const QString &src, const QString &dst, KIO::JobFlags jobFla
     }
 
     // CD into parent folder
-    const int pos = src.lastIndexOf('/');
+    const int pos = src.lastIndexOf(QLatin1Char('/'));
     if (pos >= 0) {
         if (!ftpFolder(src.left(pos + 1), false)) {
             return false;
         }
     }
 
-    QByteArray from_cmd = "RNFR ";
-    from_cmd += remoteEncoding()->encode(src.mid(pos + 1));
+    const QByteArray from_cmd = "RNFR " + remoteEncoding()->encode(src.mid(pos + 1));
     if (!ftpSendCmd(from_cmd) || (m_iRespType != 3)) {
         error(ERR_CANNOT_RENAME, src);
         return false;
     }
 
-    QByteArray to_cmd = "RNTO ";
-    to_cmd += remoteEncoding()->encode(dst);
+    const QByteArray to_cmd = "RNTO " + remoteEncoding()->encode(dst);
     if (!ftpSendCmd(to_cmd) || (m_iRespType != 2)) {
         error(ERR_CANNOT_RENAME, src);
         return false;
@@ -1240,8 +1232,7 @@ void Ftp::del(const QUrl &url, bool isfile)
         ftpFolder(remoteEncoding()->directory(url), false);    // ignore errors
     }
 
-    QByteArray cmd = isfile ? "DELE " : "RMD ";
-    cmd += remoteEncoding()->encode(url);
+    const QByteArray cmd = (isfile ? "DELE " : "RMD ") + remoteEncoding()->encode(url);
 
     if (!ftpSendCmd(cmd) || (m_iRespType != 2)) {
         error(ERR_CANNOT_DELETE, url.path());
@@ -1260,10 +1251,9 @@ bool Ftp::ftpChmod(const QString &path, int permissions)
 
     // we need to do bit AND 777 to get permissions, in case
     // we were sent a full mode (unlikely)
-    QString cmd = QLatin1String("SITE CHMOD ") + QString::number(permissions & 511, 8 /*octal*/) + ' ';
-    cmd += path;
+    const QByteArray cmd = "SITE CHMOD " + QByteArray::number(permissions & 0777/*octal*/, 8 /*octal*/) + ' ' + remoteEncoding()->encode(path);
 
-    ftpSendCmd(remoteEncoding()->encode(cmd));
+    ftpSendCmd(cmd);
     if (m_iRespType == 2) {
         return true;
     }
@@ -1292,32 +1282,32 @@ void Ftp::ftpCreateUDSEntry(const QString &filename, const FtpEntry &ftpEnt, UDS
 {
     Q_ASSERT(entry.count() == 0); // by contract :-)
 
-    entry.insert(KIO::UDSEntry::UDS_NAME, filename);
-    entry.insert(KIO::UDSEntry::UDS_SIZE, ftpEnt.size);
-    entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, ftpEnt.date.toTime_t());
-    entry.insert(KIO::UDSEntry::UDS_ACCESS, ftpEnt.access);
-    entry.insert(KIO::UDSEntry::UDS_USER, ftpEnt.owner);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, filename);
+    entry.fastInsert(KIO::UDSEntry::UDS_SIZE, ftpEnt.size);
+    entry.fastInsert(KIO::UDSEntry::UDS_MODIFICATION_TIME, ftpEnt.date.toSecsSinceEpoch());
+    entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, ftpEnt.access);
+    entry.fastInsert(KIO::UDSEntry::UDS_USER, ftpEnt.owner);
     if (!ftpEnt.group.isEmpty()) {
-        entry.insert(KIO::UDSEntry::UDS_GROUP, ftpEnt.group);
+        entry.fastInsert(KIO::UDSEntry::UDS_GROUP, ftpEnt.group);
     }
 
     if (!ftpEnt.link.isEmpty()) {
-        entry.insert(KIO::UDSEntry::UDS_LINK_DEST, ftpEnt.link);
+        entry.fastInsert(KIO::UDSEntry::UDS_LINK_DEST, ftpEnt.link);
 
         QMimeDatabase db;
-        QMimeType mime = db.mimeTypeForUrl(QUrl("ftp://host/" + filename));
+        QMimeType mime = db.mimeTypeForUrl(QUrl(QLatin1String("ftp://host/") + filename));
         // Links on ftp sites are often links to dirs, and we have no way to check
         // that. Let's do like Netscape : assume dirs generally.
         // But we do this only when the mimetype can't be known from the filename.
         // --> we do better than Netscape :-)
         if (mime.isDefault()) {
             qCDebug(KIO_FTP) << "Setting guessed mime type to inode/directory for " << filename;
-            entry.insert(KIO::UDSEntry::UDS_GUESSED_MIME_TYPE, QStringLiteral("inode/directory"));
+            entry.fastInsert(KIO::UDSEntry::UDS_GUESSED_MIME_TYPE, QStringLiteral("inode/directory"));
             isDir = true;
         }
     }
 
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, isDir ? S_IFDIR : ftpEnt.type);
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, isDir ? S_IFDIR : ftpEnt.type);
     // entry.insert KIO::UDSEntry::UDS_ACCESS_TIME,buff.st_atime);
     // entry.insert KIO::UDSEntry::UDS_CREATION_TIME,buff.st_ctime);
 }
@@ -1326,11 +1316,11 @@ void Ftp::ftpShortStatAnswer(const QString &filename, bool isDir)
 {
     UDSEntry entry;
 
-    entry.insert(KIO::UDSEntry::UDS_NAME, filename);
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, isDir ? S_IFDIR : S_IFREG);
-    entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, filename);
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, isDir ? S_IFDIR : S_IFREG);
+    entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
     if (isDir) {
-        entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
+        entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
     }
     // No details about size, ownership, group, etc.
 
@@ -1374,12 +1364,12 @@ void Ftp::stat(const QUrl &url)
     if (path.isEmpty() || path == QLatin1String("/")) {
         UDSEntry entry;
         //entry.insert( KIO::UDSEntry::UDS_NAME, UDSField( QString() ) );
-        entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
-        entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-        entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
-        entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
-        entry.insert(KIO::UDSEntry::UDS_USER, QStringLiteral("root"));
-        entry.insert(KIO::UDSEntry::UDS_GROUP, QStringLiteral("root"));
+        entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
+        entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+        entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
+        entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+        entry.fastInsert(KIO::UDSEntry::UDS_USER, QStringLiteral("root"));
+        entry.fastInsert(KIO::UDSEntry::UDS_GROUP, QStringLiteral("root"));
         // no size
 
         statEntry(entry);
@@ -1423,9 +1413,9 @@ void Ftp::stat(const QUrl &url)
         // Don't list the parent dir. Too slow, might not show it, etc.
         // Just return that it's a dir.
         UDSEntry entry;
-        entry.insert(KIO::UDSEntry::UDS_NAME, filename);
-        entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-        entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+        entry.fastInsert(KIO::UDSEntry::UDS_NAME, filename);
+        entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+        entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
         // No clue about size, ownership, group, etc.
 
         statEntry(entry);
@@ -1523,7 +1513,7 @@ void Ftp::listDir(const QUrl &url)
             realURL.setPort(m_port);
         }
         if (m_initialPath.isEmpty()) {
-            m_initialPath = '/';
+            m_initialPath = QStringLiteral("/");
         }
         realURL.setPath(m_initialPath);
         qCDebug(KIO_FTP) << "REDIRECTION to " << realURL;
@@ -1607,9 +1597,13 @@ bool Ftp::ftpOpenDir(const QString &path)
     // Pass KJob::NoError first because we don't want to emit error before we
     // have tried all commands.
     if (!ftpOpenCommand("list -la", QString(), 'I', KJob::NoError)) {
-        if (!ftpOpenCommand("list", QString(), 'I', ERR_CANNOT_ENTER_DIRECTORY)) {
-            qCWarning(KIO_FTP) << "Can't open for listing";
-            return false;
+        if (!ftpOpenCommand("list", QString(), 'I', KJob::NoError)) {
+            // Servers running with Turkish locale having problems converting 'i' letter to upper case.
+            // So we send correct upper case command as last resort.
+            if (!ftpOpenCommand("LIST -la", QString(), 'I', ERR_CANNOT_ENTER_DIRECTORY)) {
+                qCWarning(KIO_FTP) << "Can't open for listing";
+                return false;
+            }
         }
     }
     qCDebug(KIO_FTP) << "Starting of list was ok";
@@ -1620,7 +1614,7 @@ bool Ftp::ftpReadDir(FtpEntry &de)
 {
     Q_ASSERT(m_data);
 
-    // get a line from the data connecetion ...
+    // get a line from the data connection ...
     while (true) {
         while (!m_data->canReadLine() && m_data->waitForReadyRead((readTimeout() * 1000))) {}
         QByteArray data = m_data->readLine();
@@ -2105,14 +2099,12 @@ Ftp::StatusCode Ftp::ftpPut(int &iError, int iCopyFile, const QUrl &dest_url,
     }
 
     QString dest_orig = dest_url.path();
-    QString dest_part(dest_orig);
-    dest_part += QLatin1String(".part");
+    const QString dest_part = dest_orig + QLatin1String(".part");
 
     if (ftpSize(dest_orig, 'I')) {
         if (m_size == 0) {
             // delete files with zero size
-            QByteArray cmd = "DELE ";
-            cmd += remoteEncoding()->encode(dest_orig);
+            const QByteArray cmd = "DELE " + remoteEncoding()->encode(dest_orig);
             if (!ftpSendCmd(cmd) || (m_iRespType != 2)) {
                 iError = ERR_CANNOT_DELETE_PARTIAL;
                 return statusServerError;
@@ -2133,8 +2125,7 @@ Ftp::StatusCode Ftp::ftpPut(int &iError, int iCopyFile, const QUrl &dest_url,
         // file with extension .part exists
         if (m_size == 0) {
             // delete files with zero size
-            QByteArray cmd = "DELE ";
-            cmd += remoteEncoding()->encode(dest_part);
+            const QByteArray cmd = "DELE " + remoteEncoding()->encode(dest_part);
             if (!ftpSendCmd(cmd) || (m_iRespType != 2)) {
                 iError = ERR_CANNOT_DELETE_PARTIAL;
                 return statusServerError;
@@ -2217,8 +2208,7 @@ Ftp::StatusCode Ftp::ftpPut(int &iError, int iCopyFile, const QUrl &dest_url,
             // Remove if smaller than minimum size
             if (ftpSize(dest, 'I') &&
                     (processed_size < config()->readEntry("MinimumKeepSize", DEFAULT_MINIMUM_KEEP_SIZE))) {
-                QByteArray cmd = "DELE ";
-                cmd += remoteEncoding()->encode(dest);
+                const QByteArray cmd = "DELE " + remoteEncoding()->encode(dest);
                 (void) ftpSendCmd(cmd);
             }
         }
@@ -2263,9 +2253,7 @@ bool Ftp::ftpSize(const QString &path, char mode)
         return false;
     }
 
-    QByteArray buf;
-    buf = "SIZE ";
-    buf += remoteEncoding()->encode(path);
+    const QByteArray buf = "SIZE " + remoteEncoding()->encode(path);
     if (!ftpSendCmd(buf) || (m_iRespType != 2)) {
         return false;
     }
@@ -2285,9 +2273,7 @@ bool Ftp::ftpSize(const QString &path, char mode)
 
 bool Ftp::ftpFileExists(const QString &path)
 {
-    QByteArray buf;
-    buf = "SIZE ";
-    buf += remoteEncoding()->encode(path);
+    const QByteArray buf = "SIZE " + remoteEncoding()->encode(path);
     if (!ftpSendCmd(buf) || (m_iRespType != 2)) {
         return false;
     }
@@ -2319,8 +2305,7 @@ bool Ftp::ftpDataMode(char cMode)
         return true;
     }
 
-    QByteArray buf = "TYPE ";
-    buf += cMode;
+    const QByteArray buf = QByteArrayLiteral("TYPE ") + cMode;
     if (!ftpSendCmd(buf) || (m_iRespType != 2)) {
         return false;
     }
@@ -2332,8 +2317,8 @@ bool Ftp::ftpFolder(const QString &path, bool bReportError)
 {
     QString newPath = path;
     int iLen = newPath.length();
-    if (iLen > 1 && newPath[iLen - 1] == '/') {
-        newPath.truncate(iLen - 1);
+    if (iLen > 1 && newPath[iLen - 1] == QLatin1Char('/')) {
+        newPath.chop(1);
     }
 
     qCDebug(KIO_FTP) << "want" << newPath << "has" << m_currentPath;
@@ -2341,8 +2326,7 @@ bool Ftp::ftpFolder(const QString &path, bool bReportError)
         return true;
     }
 
-    QByteArray tmp = "cwd ";
-    tmp += remoteEncoding()->encode(newPath);
+    const QByteArray tmp = "cwd " + remoteEncoding()->encode(newPath);
     if (!ftpSendCmd(tmp)) {
         return false;    // connection failure
     }
@@ -2374,16 +2358,10 @@ void Ftp::copy(const QUrl &src, const QUrl &dest, int permissions, KIO::JobFlags
         sCopyFile = src.toLocalFile();
         qCDebug(KIO_FTP) << "local file" << sCopyFile << "-> ftp" << dest.path();
         cs = ftpCopyPut(iError, iCopyFile, sCopyFile, dest, permissions, flags);
-        if (cs == statusServerError) {
-            sCopyFile = dest.toString();
-        }
     } else if (!bSrcLocal && bDestLocal) {          // Ftp -> File
         sCopyFile = dest.toLocalFile();
         qCDebug(KIO_FTP) << "ftp" << src.path() << "-> local file" << sCopyFile;
         cs = ftpCopyGet(iError, iCopyFile, sCopyFile, src, permissions, flags);
-        if (cs == statusServerError) {
-            sCopyFile = src.toString();
-        }
     } else {
         error(ERR_UNSUPPORTED_ACTION, QString());
         return;
@@ -2394,10 +2372,12 @@ void Ftp::copy(const QUrl &src, const QUrl &dest, int permissions, KIO::JobFlags
         QT_CLOSE(iCopyFile);
     }
     ftpCloseCommand();                        // must close command!
-    if (iError) {
-        error(iError, sCopyFile);
-    } else {
-        finished();
+    if (cs != statusServerError) {
+        if (iError) {
+            error(iError, sCopyFile);
+        } else {
+            finished();
+        }
     }
 }
 
@@ -2542,8 +2522,9 @@ Ftp::StatusCode Ftp::ftpCopyGet(int &iError, int &iCopyFile, const QString &sCop
             if (dt.isValid()) {
                 qCDebug(KIO_FTP) << "Updating modified timestamp to" << mtimeStr;
                 struct utimbuf utbuf;
-                utbuf.actime = sPartInfo.lastRead().toTime_t(); // access time, unchanged
-                utbuf.modtime = dt.toTime_t(); // modification time
+                info.refresh();
+                utbuf.actime = info.lastRead().toSecsSinceEpoch(); // access time, unchanged
+                utbuf.modtime = dt.toSecsSinceEpoch(); // modification time
                 ::utime(QFile::encodeName(sCopyFile).constData(), &utbuf);
             }
         }
@@ -2607,7 +2588,7 @@ void Ftp::proxyAuthentication(const QNetworkProxy &proxy, QAuthenticator *authen
     if (!haveCachedCredentials || m_socketProxyAuth) {
         // Save authentication info if the connection succeeds. We need to disconnect
         // this after saving the auth data (or an error) so we won't save garbage afterwards!
-        connect(m_control, SIGNAL(connected()), this, SLOT(saveProxyAuthentication()));
+        connect(m_control, &QAbstractSocket::connected, this, &Ftp::saveProxyAuthentication);
         //### fillPromptInfo(&info);
         info.prompt = i18n("You need to supply a username and a password for "
                            "the proxy server listed below before you are allowed "
@@ -2639,7 +2620,7 @@ void Ftp::proxyAuthentication(const QNetworkProxy &proxy, QAuthenticator *authen
 void Ftp::saveProxyAuthentication()
 {
     qCDebug(KIO_FTP);
-    disconnect(m_control, SIGNAL(connected()), this, SLOT(saveProxyAuthentication()));
+    disconnect(m_control, &QAbstractSocket::connected, this, &Ftp::saveProxyAuthentication);
     Q_ASSERT(m_socketProxyAuth);
     if (m_socketProxyAuth) {
         qCDebug(KIO_FTP) << "-- realm:" << m_socketProxyAuth->realm() << "user:" << m_socketProxyAuth->user();
