@@ -29,17 +29,17 @@
 #include <assert.h>
 #include <limits.h>
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QMutex>
-#include <QtCore/QRegExp>
-#include <QtCore/QTimer>
-#include <QtCore/QDir>
-#include <QtCore/QDirIterator>
-#include <QtCore/QFile>
-#include <QtCore/QTextStream>
-#include <QtCore/QThread>
-#include <QtCore/QUrl>
-#include <QtCore/QDebug>
+#include <QCoreApplication>
+#include <QMutex>
+#include <QRegExp>
+#include <QTimer>
+#include <QDir>
+#include <QDirIterator>
+#include <QFile>
+#include <QTextStream>
+#include <QThread>
+#include <QUrl>
+#include <QDebug>
 #include <QMimeDatabase>
 #include <QProcessEnvironment>
 #include <qplatformdefs.h> // QT_LSTAT, QT_STAT, QT_STATBUF
@@ -54,7 +54,6 @@
 #include <KUser>
 #include <kio_widgets_debug.h>
 
-#include <qplatformdefs.h>
 #include <time.h>
 #include <kconfiggroup.h>
 
@@ -62,6 +61,7 @@
 #include <qt_windows.h>
 #else
 #include <sys/param.h>
+#include <pwd.h>
 #endif
 
 static bool expandTilde(QString &);
@@ -109,7 +109,7 @@ static int initialWaitDuration()
 class KUrlCompletionPrivate
 {
 public:
-    KUrlCompletionPrivate(KUrlCompletion *parent)
+    explicit KUrlCompletionPrivate(KUrlCompletion *parent)
         : q(parent),
           url_auto_completion(true),
           userListThread(nullptr),
@@ -160,7 +160,7 @@ public:
                      const QString &filter = QString(),
                      bool no_hidden = false);
 
-    KUrlCompletion *q;
+    KUrlCompletion * const q;
     QList<QUrl> list_urls;
 
     bool onlyLocalProto;
@@ -273,9 +273,10 @@ public:
     {}
 
 protected:
-    void run() Q_DECL_OVERRIDE
+    void run() override
     {
-        static const QChar tilde = '~';
+#ifndef Q_OS_ANDROID
+        const QChar tilde = QLatin1Char('~');
 
         // we don't need to handle prepend here, right? ~user is always at pos 0
         assert(m_prepend.isEmpty());
@@ -289,13 +290,13 @@ protected:
         ::endpwent();
 #else
         //currently terminationRequested is ignored on Windows
-        QStringList allUsers = KUser::allUserNames();
-        Q_FOREACH(const QString& s, allUsers) {
+        const QStringList allUsers = KUser::allUserNames();
+        for (const QString& s : allUsers) {
             addMatch(tilde + s);
         }
 #endif
         addMatch(QString(tilde));
-
+#endif
         done();
     }
 };
@@ -322,7 +323,7 @@ public:
         m_appendSlashToDir(appendSlashToDir)
     {}
 
-    void run() Q_DECL_OVERRIDE;
+    void run() override;
 
 private:
     QStringList m_dirList;
@@ -837,14 +838,12 @@ bool KUrlCompletionPrivate::envCompletion(const KUrlCompletionPrivate::MyURL &ur
         q->clear();
 
         QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-        QStringList keys = env.keys();
-
-        QString dollar = QStringLiteral("$");
+        const QStringList keys = env.keys();
 
         QStringList l;
-
-        Q_FOREACH(const QString &key, keys) {
-            l.append(prepend + dollar + key);
+        l.reserve(keys.size());
+        for (const QString &key : keys) {
+            l.append(prepend + QLatin1Char('$') + key);
         }
 
         addMatches(l);
@@ -880,14 +879,8 @@ bool KUrlCompletionPrivate::exeCompletion(const KUrlCompletionPrivate::MyURL &ur
 
     if (!url.file().isEmpty()) {
         // $PATH
-        // ### maybe Qt should have a QDir::pathSeparator() to avoid ifdefs..
-#ifdef Q_OS_WIN
-#define KPATH_SEPARATOR ';'
-#else
-#define KPATH_SEPARATOR ':'
-#endif
         dirList = QString::fromLocal8Bit(qgetenv("PATH")).split(
-                      KPATH_SEPARATOR, QString::SkipEmptyParts);
+                      QDir::listSeparator(), QString::SkipEmptyParts);
 
         QStringList::Iterator it = dirList.begin();
 
@@ -956,7 +949,7 @@ bool KUrlCompletionPrivate::fileCompletion(const KUrlCompletionPrivate::MyURL &u
         // current directory
         QString dirToAdd = cwd.toLocalFile();
         if (!directory.isEmpty()) {
-            if (!dirToAdd.endsWith('/')) {
+            if (!dirToAdd.endsWith(QLatin1Char('/'))) {
                 dirToAdd.append(QLatin1Char('/'));
             }
             dirToAdd.append(directory);
@@ -1016,7 +1009,7 @@ bool KUrlCompletionPrivate::urlCompletion(const KUrlCompletionPrivate::MyURL &ur
     }
 
     // url is malformed
-    if (!url_dir.isValid()) {
+    if (!url_dir.isValid() || url.scheme().isEmpty()) {
         return false;
     }
 
@@ -1147,6 +1140,7 @@ QString KUrlCompletionPrivate::listDirectories(
     QStringList::ConstIterator it = dirList.constBegin();
     QStringList::ConstIterator end = dirList.constEnd();
 
+    url_list.reserve(dirList.size());
     for (; it != end; ++it) {
         url_list.append(QUrl(*it));
     }
@@ -1234,7 +1228,7 @@ void KUrlCompletionPrivate::_k_slotEntries(KIO::Job *, const KIO::UDSEntryList &
             continue;
         }
 
-        if (filter_len != 0 && entry_name.left(filter_len) != filter) {
+        if (filter_len != 0 && entry_name.leftRef(filter_len) != filter) {
             continue;
         }
 
@@ -1472,9 +1466,9 @@ static bool expandEnv(QString &text)
             //
             if (pos2 >= 0) {
                 int len = pos2 - pos;
-                QString key = text.mid(pos + 1, len - 1);
+                const QStringRef key = text.midRef(pos + 1, len - 1);
                 QString value =
-                    QString::fromLocal8Bit(qgetenv(key.toLocal8Bit()));
+                    QString::fromLocal8Bit(qgetenv(key.toLocal8Bit().constData()));
 
                 if (!value.isEmpty()) {
                     expanded = true;
